"""This is main graspfile module for manipulating files from TICRA Tools, GRASP and CHAMP

Submodules
==========

.. autosummary::
    :toctree: _autosummary

    cut
    grid
"""

__version__ = '0.3.0'

__all__ = ['cut', 'grid', 'torfile', 'torparser']
